import { strict as assert } from 'node:assert';

const _meta = { draftSAdmonition: true };

function jsx(version, ast) {
  if (version === 3) {
    return {
      lang: 'mdx-code-block',
      type: 'code',
      _meta,
      ...ast,
    };
  }

  return {
    type: 'jsx',
    _meta,
    ...ast,
  };
}

export function draftSAdmonition({
  text = '',
  type = '_meta',
  title = 'Drafted',
  version = 2,
} = {}) {
  assert(typeof title === 'string', new TypeError('`title` should be string'));
  assert(typeof type === 'string', new TypeError('`type` should be string'));
  assert(typeof text === 'string', new TypeError('`text` should be string'));
  assert([2, 3].includes(version), new TypeError('`version` should be 2 or 3'));

  return (tree) => {
    if (text && !tree.children.some((node) => node._meta?.draftSAdmonition)) {
      const index =
        tree.children.findIndex(
          (node) => node.type === 'heading' && node.depth === 1,
        ) || 0;

      tree.children.splice(
        index + 1,
        0,
        jsx(version, {
          value: '<div hidden={!frontMatter.draft}>',
        }),
        {
          _meta,
          ...(version === 2
            ? {
                type: 'admonitionHTML',
                data: {
                  hName: 'admonition',
                  hProperties: { title, type },
                },
              }
            : {
                type: 'containerDirective',
                name: type,
              }),
          children: [
            {
              type: 'paragraph',
              children: [{ type: 'text', value: text }],
            },
          ],
        },
        jsx(version, {
          value: '</div>',
        }),
      );
    }
  };
}
